import React, {
  PropTypes, Component,
} from 'react';

import {
  ScrollView, View,
} from 'react-native';

import Select from '../Select';
import { noop } from '../helper';

const propTypes = {
  children: PropTypes.node,
  mask: PropTypes.oneOfType([
    PropTypes.bool,
    PropTypes.element,
  ]),
  onChange: PropTypes.func,
  style: View.propTypes.style,
};

const defaultProps = {
  children: null,
  mask: false,
  onChange: noop,
  style: null,
};

class Picker extends Component {
  onScroll = ({ nativeEvent }) => {
    this.scrollPoint = nativeEvent.contentOffset;
    if (this.scrollFake) {
      this.scrollFake.scrollTo({
        ...this.scrollPoint,
        animated: false,
      });
    }
  }

  onScrollEnd = () => {
    this.scrollFinish = true;
    const { onChange, children } = this.props;
    const { y } = this.scrollPoint;
    let index = Math.round(y / this.itemHeight);
    if (this.scrollTarget) {
      this.scrollTarget.scrollTo({
        ...this.scrollPoint,
        y: this.itemHeight * index,
        animated: true,
      });
    }
    if (index >= children.length) index = children.length - 1;
    if (index < 0) index = 0;
    onChange(children[index].props.value);
  }

  onScrollBeginDrag = () => {
    this.scrollFinish = false;
    this.scrollMomentum = false;
    this.scrollDraging = true;
    this.clearScrollTimer();
    this.clearMomentumTimer();
  };

  onScrollEndDrag = () => {
    this.scrollDraging = false;
    this.setScrollTimer();
  };

  onMomentumScrollBegin = () => {
    this.scrollMomentum = true;
    this.clearScrollTimer();
  };

  onMomentumScrollEnd = () => {
    this.scrollMomentum = false;
    this.setMomentumTimer();
  };

  onScrollReady = (scroll) => {
    this.scrollTarget = scroll;
  }

  onFakeReady = (scroll) => {
    this.scrollFake = scroll;
  }

  setScrollTimer = () => {
    this.clearScrollTimer();
    this.scrollTimer = setTimeout(() => {
      if (!this.scrollMomentum) {
        this.onScrollEnd();
      }
    }, 16);
  }

  setMomentumTimer = () => {
    this.clearMomentumTimer();
    this.momentumTimer = setTimeout(() => {
      if (!this.scrollFinish) {
        this.onScrollEnd();
      }
    }, 16);
  }

  getContainerStyle = () => (
    this.containerHeight ? {
      height: this.containerHeight,
    } : {}
  )

  getSelectStyle = () => (
    this.paddingVertical ? {
      paddingVertical: this.paddingVertical,
    } : {}
  )

  getItemStyle = (style) => {
    let style$ = style;
    if (!Array.isArray(style$)) {
      style$ = [style$];
    }

    style$ = [
      {
        alignItems: 'center',
        justifyContent: 'center',
        height: 36,
      },
      ...style$,
    ];

    return [
      ...style$,
      this.itemHeight
        ? {
          height: this.itemHeight,
        }
        : {},
    ];
  }

  clearScrollTimer = () => {
    if (this.scrollTimer) {
      clearTimeout(this.scrollTimer);
      this.scrollTimer = null;
    }
  }

  clearMomentumTimer = () => {
    if (this.momentumTimer) {
      clearTimeout(this.momentumTimer);
      this.momentumTimer = null;
    }
  }

  containerHeight = 0
  paddingVertical = 0
  itemHeightList = []
  itemHeight = 0
  initCount = 0

  scrollPoint = { x: 0, y: 0 };
  scrollTarget = null;
  scrollDraging = false;
  scrollMomentum = false;
  scrollFinish = false;
  scrollFake = null;

  initContainer = ({ nativeEvent }) => {
    this.containerHeight = nativeEvent.layout.height;
    this.initReady();
  }

  initItem = ({ nativeEvent }, index) => {
    this.itemHeightList[index] = nativeEvent.layout.height;
    this.initReady();
  }

  initReady = () => {
    if (this.initCount === this.props.children.length) {
      this.itemHeight = Math.max(...this.itemHeightList);
      this.paddingVertical = Math.floor((this.containerHeight - this.itemHeight) / 2);
      this.containerHeight = (this.paddingVertical * 2) + this.itemHeight;
      this.forceUpdate();
    }
    this.initCount += 1;
  }

  render() {
    const { initContainer, initItem } = this;
    const { children, mask, style } = this.props;
    if (!mask) {
      return (
        <ScrollView
          onLayout={initContainer}
          showsVerticalScrollIndicator={false}
          style={[style, this.getContainerStyle()]}
          ref={this.onScrollReady}
          scrollEventThrottle={16}
          onScroll={this.onScroll}
          onScrollBeginDrag={this.onScrollBeginDrag}
          onScrollEndDrag={this.onScrollEndDrag}
          onMomentumScrollBegin={this.onMomentumScrollBegin}
          onMomentumScrollEnd={this.onMomentumScrollEnd}
        >
          <Select
            style={this.getSelectStyle()}
            onScroll={this.onScroll}
            onScrollEnd={this.onScrollEnd}
            onScrollReady={this.onScrollReady}
          >
            {children.map((item, index) => {
              const key = index;
              return (
                <item.type
                  {...item.props}
                  key={key}
                  onLayout={event => initItem(event, index)}
                  style={this.getItemStyle(item.props.style)}
                />
              );
            })}
          </Select>
        </ScrollView>
      );
    }

    return (
      <View style={style}>
        <ScrollView
          onLayout={initContainer}
          showsVerticalScrollIndicator={false}
          style={this.getContainerStyle()}
          ref={this.onFakeReady}
        >
          <Select style={this.getSelectStyle()}>
            {children.map((item, index) => {
              const key = index;
              return (
                <item.type
                  {...item.props}
                  key={key}
                  onLayout={event => initItem(event, index)}
                  style={this.getItemStyle(item.props.style)}
                />
              );
            })}
          </Select>
        </ScrollView>
        <View position="absolute" top={0} right={0} bottom={0} left={0} justifyContent="space-between">
          {mask === true ? ['up', 'down'].map((item, index) => {
            const key = index;
            return <View key={key} height={this.paddingVertical} backgroundColor="rgba(246,246,246,.8)" />;
          }) : mask}
        </View>
        <ScrollView
          style={[this.getContainerStyle(), { opacity: 0, position: 'absolute', top: 0, right: 0, bottom: 0, left: 0 }]}
          ref={this.onScrollReady}
          scrollEventThrottle={16}
          onScroll={this.onScroll}
          onScrollBeginDrag={this.onScrollBeginDrag}
          onScrollEndDrag={this.onScrollEndDrag}
          onMomentumScrollBegin={this.onMomentumScrollBegin}
          onMomentumScrollEnd={this.onMomentumScrollEnd}
        >
          <Select style={this.getSelectStyle()}>
            {children.map((item, index) => {
              const key = index;
              return (
                <item.type
                  {...item.props}
                  key={key}
                  style={this.getItemStyle(item.props.style)}
                />
              );
            })}
          </Select>
        </ScrollView>
      </View>
    );
  }
}

Picker.propTypes = propTypes;
Picker.defaultProps = defaultProps;

export default Picker;
