import React, {
  Component, PropTypes, ViewStylePropTypes,
} from 'react';

import {
  Dimensions, Text, View, ScrollView,
} from 'react-native';

import DefaultTabBar from '../TabBar';
import SceneComponent from './Scene';
import styles from './style';

const DEVICE_WIDTH = Dimensions.get('window').width;

const propTypes = {
  tabBarPosition: PropTypes.oneOf(['top', 'bottom']),
  initialPage: PropTypes.number,
  page: PropTypes.number,
  onChangeTab: PropTypes.func,
  renderTabBar: PropTypes.func,
  activeTextStyle: Text.propTypes.style,
  defaultTextStyle: Text.propTypes.style,
  tabBarStyle: ViewStylePropTypes || (View.propTypes && View.propTypes.style),
  tabUnderLineStyle: ViewStylePropTypes || (View.propTypes && View.propTypes.style), // 下划线背景
  children: PropTypes.array,
  scrollEnabled: PropTypes.bool,
};
const defaultProps = {
  scrollEnabled: true,
  tabBarPosition: 'top',
  initialPage: 0,
  page: 0,
  onChangeTab: () => {},
  tabBarStyle: null,
  tabUnderLineStyle: null,
  defaultTextStyle: styles.defaultTextStyle,
  activeTextStyle: styles.activeTextStyle,
  style: null,
  renderTabBar: null,
  children: [],
};

class TabScene extends Component {
  constructor(props) {
    super(props);
    this.onActive = [];
    this.onDeactive = [];
    this.state = {
      currentPage: props.initialPage,
      containerWidth: DEVICE_WIDTH,
    };
  }

  componentWillReceiveProps = (props) => {
    if (props.page >= 0 && props.page !== this.state.currentPage) {
      this.goToScene(props.page);
    }
  }

  get tabBarProps() {
    return {
      goToScene: this.goToScene,
      tabs: this.getChildren().map(child => child.props.tabLabel),
      activeTab: this.state.currentPage,
      activeTextStyle: this.props.activeTextStyle,
      defaultTextStyle: this.props.defaultTextStyle,
      tabUnderLineStyle: this.props.tabUnderLineStyle,
      tabBarStyle: this.props.tabBarStyle,
    };
  }
  getChildren = (children = this.props.children) => React.Children.map(children, child => child);

  scrollToItem(itemIndex) {
    const scrollToX = itemIndex * DEVICE_WIDTH;
    if (this.props.scrollEnabled && this.screen) {
      this.screen.scrollTo({ y: 0, x: scrollToX, animated: false });
    }
  }

  goToScene = (pageNumber) => {
    this.scrollToItem(pageNumber);
    const currentPage = this.state.currentPage;
    this.updateSceneKeys({
      page: pageNumber,
      callback: this.handleChangeTab.bind(this, currentPage, pageNumber),
    });
    if (typeof this.onActive[pageNumber] === 'function') { this.onActive[pageNumber](); }
    if (typeof this.onDeactive[currentPage] === 'function') { this.onDeactive[currentPage](); }
  }

  updateSceneKeys = ({ page, callback = () => {} }) => {
    this.setState({ currentPage: page }, callback);
  }

  handleChangeTab = (prevPage, currentPage) => {
    this.props.onChangeTab({
      i: currentPage,
      ref: this.getChildren()[currentPage],
      from: prevPage,
    });
  }

  scrollEnded(event) {
    const position = event.nativeEvent.contentOffset.x;
    const currentPage = position / DEVICE_WIDTH;
    if (currentPage !== this.state.currentPage) { this.goToScene(currentPage); }
  }

  bindOnActive(key, callback) {
    this.onActive[key] = callback;
  }
  bindEvent(obj, key, callback) {
    const arr = obj;
    arr[key] = callback;
  }

  renderScrollableContent = () => this.getChildren().map(((child, index) => {
    const key = index;
    if (child.props.onActive) { this.bindEvent(this.onActive, key, child.props.onActive); }
    if (child.props.onDeactive) { this.bindEvent(this.onDeactive, key, child.props.onDeactive); }
    return (
      <SceneComponent
        key={child.key}
        style={child.props.style}
      >
        {child}
      </SceneComponent>);
  }));

  renderTabBar = (props) => {
    if (this.props.renderTabBar === false) {
      return null;
    } else if (this.props.renderTabBar) {
      return React.cloneElement(this.props.renderTabBar(props), props);
    }
    return <DefaultTabBar ref={(ref) => { this.tabs = ref; }} style={{ height: 30 }} {...props} />;
  }

  render() {
    return (
      <View style={styles.container}>
        {this.props.tabBarPosition === 'top' && this.renderTabBar(this.tabBarProps)}
        <View style={{ flex: 1 }}>
          <ScrollView
            ref={(ref) => { this.screen = ref; }}
            horizontal
            scrollEnabled
            pagingEnabled
            removeClippedSubviews
            scrollEventThrottle={500} // 每0.5S被调用1次滑动监听事件
            showsHorizontalScrollIndicator={false}
            automaticallyAdjustContentInsets={false}
            onMomentumScrollEnd={event => this.scrollEnded(event)}
          >
            {this.renderScrollableContent()}
          </ScrollView>
        </View>
        {this.props.tabBarPosition === 'bottom' && this.renderTabBar(this.tabBarProps)}
      </View>
    );
  }
}

TabScene.propTypes = propTypes;
TabScene.defaultProps = defaultProps;

export default TabScene;
